import sys
import unittest
from unittest.mock import patch
import mysql.connector
from mysql.connector import errorcode
from src.csv_to_db_converter import crud_operation_db

MYSQL_USER = "root"
MYSQL_PASSWORD = "saatwik"
MYSQL_DB = 'Fake_DB'
MYSQL_HOST = "localhost"


class My_DB(unittest.TestCase):

    @classmethod
    def setUpClass(cls):

        cnx = mysql.connector.connect(
            host=MYSQL_HOST,
            user=MYSQL_USER,
            password=MYSQL_PASSWORD,

        )
        cursor = cnx.cursor()

        # drop database if it already exists
        try:
            cursor.execute("DROP DATABASE {}".format(MYSQL_DB))
            cursor.close()
            print("DB dropped")
        except mysql.connector.Error as err:
            print("{}{}".format(MYSQL_DB, err))

        cursor = cnx.cursor()
        try:
            cursor.execute(
                "CREATE DATABASE {}".format(MYSQL_DB))
        except mysql.connector.Error as err:
            print("Failed creating database: {}".format(err))
            sys.exit(1)
        cnx.database = MYSQL_DB

        query = """CREATE TABLE test_table (
                  id varchar(50) PRIMARY KEY,
                  title text,
                  original_title text,
                  original_title_romanised text,
                  image text,
                  movie_banner text, 
                  description text,
                  director text,
                  producer text,
                  release_date int,
                  running_time int,
                  rt_score int,
                  people text,
                  species text,
                  locations text, 
                  vehicles text,
                  url text
                )"""
        try:
            cursor.execute(query)
            cnx.commit()
        except mysql.connector.Error as err:
            if err.errno == errorcode.ER_TABLE_EXISTS_ERROR:
                print("test_table already exists.")
            else:
                print(err.msg)
        else:
            print("OK")

        insert_data_query = """INSERT INTO test_table VALUES
                            ('id_1','vroom',
                            'vroom_orginal','vroom_romanised',
                             'image.jpg','banner.jpg',
                             'it is a movie','ramaj','nuthan',
                             2021,100,99,'some,people,movie','human'
                             ,'anylocation','car,plane','aurl'),
                             ('id_2','grom',
                            'grom_orginal','grom_romanised',
                             'image1.jpg','banner1.jpg',
                             'it is another movie','ramanuj','nuthanil',
                             2001,90,99,'some,people,movie','human'
                             ,'anylocation','car,plane','aurl')
                            """
        try:
            cursor.execute(insert_data_query)
            cnx.commit()
        except mysql.connector.Error as err:
            print("Data insertion to test_table failed \n" + err)
        cursor.close()
        cnx.close()

        testconfig = {
            'host': MYSQL_HOST,
            'user': MYSQL_USER,
            'password': MYSQL_PASSWORD,

        }
        cls.mock_db_config = patch.dict(crud_operation_db.config, testconfig)

    @classmethod
    def tearDownClass(cls):

        cnx = mysql.connector.connect(
            host=MYSQL_HOST,
            user=MYSQL_USER,
            password=MYSQL_PASSWORD,
        )
        cursor = cnx.cursor()

        # drop test database
        try:
            cursor.execute("DROP DATABASE {}".format(MYSQL_DB))
            cnx.commit()
            cursor.close()
        except mysql.connector.Error as err:
            print("Database {} does not exists. Dropping db failed".format(MYSQL_DB))
        cnx.close()



"""This module converts any given file data into db table that can be stored for later work"""
import logging
import os.path
from mysql import connector
import pandas as pd

logging.basicConfig(filename='CRUD_operation.log',
                    level=logging.INFO,
                    format='%(asctime)s: %(levelname)s:'
                           ' %(filename)s->'
                           ' %(funcName)s->'
                           ' Line %(lineno)d-> %(message)s')

PYTHON_TO_SQL_TYPE = {'string': 'TEXT',
                      'floating': 'FLOAT',
                      'integer': 'BIGINT',
                      'boolean': 'BOOLEAN',
                      'date': 'DATE',
                      'time': 'TIME'}


def csv_to_db(filename=None, primary_key_field: str = None, pk_field_dtype=None):
    """
    Function use to convert your csv file data into database table.
    Takes the CSV file as input and Primary field to set in the dataTable.

    :param primary_key_field: Name of the column in your csv
                              file that should be used as the primary key \
                              in the db table.
    :param pk_field_dtype: Set the data type of the field that should be used
                            as primary key.
    :param filename: Name of the file which contains the data to be stored
                    in the db table. The filename will also be used as the
                    name of the db table as well.
    :return: None
    """
    try:
        if filename is not None and \
                os.path.isfile(filename) and \
                os.path.splitext(filename)[-1] == '.csv':
            table_name = os.path.splitext(filename)[0].lower()

            csv_dataframe = pd.read_csv(filename, keep_default_na=False)

            column_inf_dtype = [pd.api.types.infer_dtype(csv_dataframe[i], skipna=True)
                                for i in csv_dataframe.columns]
            mysql_dtype_column = [PYTHON_TO_SQL_TYPE[i] for i in column_inf_dtype]
            columns_in_df = [i.replace('.', '_') for i in list(csv_dataframe.columns)]
            csv_dataframe.columns = columns_in_df
            column_with_dtype = [(i[0] + " " + i[1])
                                 for i in zip(csv_dataframe[columns_in_df]
                                              , mysql_dtype_column)]
            if primary_key_field is not None and primary_key_field != '' \
                    and pk_field_dtype is not None and pk_field_dtype != '' \
                    and primary_key_field in list(csv_dataframe.columns):
                for i in range(len(column_with_dtype)):
                    if primary_key_field in column_with_dtype[i]:
                        column_with_dtype[i] = "{} {} PRIMARY KEY".format(primary_key_field, pk_field_dtype)
                        break

            conn = connector.connect(host='localhost',
                                     user='root',
                                     password='saatwik')

            if conn.is_connected():
                cursor = conn.cursor()
                cursor.execute("CREATE DATABASE IF NOT EXISTS fileupload")
                cursor.execute("USE fileupload")
                cursor.execute(f'DROP TABLE IF EXISTS {table_name}')
                cursor.execute(f'CREATE TABLE IF NOT EXISTS {table_name} ({",".join(column_with_dtype)})')
                for i in csv_dataframe.values.tolist():
                    cursor.execute(f'INSERT INTO {table_name} VALUES {tuple(i)}')
                conn.commit()
                conn.close()

    except connector.ProgrammingError as pro_err:
        logging.error("%s: %s", pro_err.__class__.__name__, pro_err)
        logging.error("arguments value are \nfilename:%s,"
                      "\nprimary_key_field:%s,\npk_field_dtype:%s "
                      , filename, primary_key_field, pk_field_dtype)
    except connector.IntegrityError as integrity_err:
        logging.error("Error occured %s: %s",
                      integrity_err.__class__.__name__, integrity_err)
    except ValueError as val_err:
        logging.error("%s: %s", val_err.__class__.__name__, val_err)
        logging.error("arguments value are \nfilename:%s,"
                      "\nprimary_key_field:%s,\npk_field_dtype:%s "
                      , filename, primary_key_field, pk_field_dtype)
    except Exception as exc:
        logging.error("%s: %s", exc.__class__.__name__, exc)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="csv-to-db-converter-saatwikmehta",
    version="0.0.4",
    author="Saatwik Mehta",
    author_email="saatwikmehta@gmail.com",
    description="This package is useful for someone who wants to make changes inside his CSV files."
                "It creates a local server that provides the functionality of file upload to db"
                "Reading the data inside the db table created using the name of the file and "
                "applying CRUD operations as well."
                ,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Saatwik-Mehta/csv_to_db_pkg",
    project_urls={
        "Bug Tracker": "https://github.com/Saatwik-Mehta/csv_to_db_pkg/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
)


""" This module contains useful CRUD operation function that anyone
can use for their database table. It mainly required the nme of the DB and dataset."""

import logging
from typing import List

from mysql import connector

logging.basicConfig(filename='CRUD_operation.log',
                    level=logging.INFO,
                    format='%(asctime)s: %(levelname)s:'
                           ' %(filename)s->'
                           ' %(funcName)s->'
                           ' Line %(lineno)d-> %(message)s')

config = {
    'host': 'localhost',
    'user': 'root',
    'password': 'saatwik',

}


def view_db_data(db_name: str = None, db_table: str = None, filter_data: dict = None):
    """

    This function shows you the data inside the database table.
    It provides you the functionality to filter the data also.

    :param filter_data: A dict variable which is used to
                        filter the data of the database table.

                        *-> Use keyname -> 'columns' to select the specific columns
                        from the db table.

                        Eg: filter_data = {'columns':'col1' or 'columns': 'col1,col2....'}

                        *-> Use keyname -> 'where' to use conditional
                        selection from the db table.
                        Eg: filter_data = {'where':'age>25' or
                        'where':'age>25 and/or salary<30000'}
    :param db_name: Database name which is containing the data table.
    :param db_table: Data Table name from which the data will be retrieved
    :return: string containing HTML table tag
    """
    try:
        if db_name is not None and db_table is not None:
            conn = connector.connect(**config)
            html_data = ''
            if conn.is_connected():
                cursor = conn.cursor()

                if filter_data is not None and isinstance(filter_data, dict):
                    if 'columns' in filter_data.keys() and filter_data['columns'] != '':
                        if 'where' in filter_data.keys():
                            cursor.execute(f'SELECT {filter_data["columns"]} FROM '
                                           f'{db_name}.{db_table} WHERE {filter_data["where"]}')
                        else:
                            cursor.execute(f'SELECT {filter_data["columns"]} '
                                           f'FROM {db_name}.{db_table}')
                    else:
                        if 'where' in filter_data.keys():
                            cursor.execute(f'SELECT * FROM {db_name}.{db_table} '
                                           f'WHERE {filter_data["where"]}')

                        else:
                            cursor.execute(f'SELECT * FROM {db_name}.{db_table}')
                else:
                    cursor.execute(f'SELECT * FROM {db_name}.{db_table}')
                columns = cursor.column_names
                db_data = cursor.fetchall()

                html_data += """<table><tr>"""
                for i in range(len(columns)):
                    html_data += f"""<th id="thead_{i}">{columns[i]}</th>"""
                html_data += """</tr>"""
                for rows in range(len(db_data)):
                    html_data += """<tr>"""
                    for item in range(len(db_data[rows])):
                        html_data += f"""<td id="td_{rows}.{item}">{db_data[rows][item]}</td>"""
                    html_data += """</tr>"""
                html_data += """</table>"""
            return html_data
        return f"Either DataBase {db_name} or Table {db_table} doesn't exist!"
    except connector.ProgrammingError as prog_err:
        logging.error('%s: %s', prog_err.__class__.__name__, prog_err)
        raise

    except connector.Error as err:
        logging.error('%s: %s', err.__class__.__name__, err)
        raise
    finally:
        conn.close()


def delete_db_data(db_name: str = None, db_table: str = None, expression=None):
    """
    This function can be used to delete a specific column from the database table.
    It requires the column name and value from the row which needs to be deleted.
    :param db_name: Database name which is containing the data table.
    :param db_table: Data Table name from which the data will be retrieved
    :param expression: A list variable that takes column name
                        and the value as string to be deleted.
                        Eg: expression=[column: str, value: str] ->
                            ['id','thi-id-123'] or ['id','2']
    :return: msg-> 'Deleted successfully' or error->"Programming error"
    """
    try:
        if db_name is not None and db_table is not None:
            conn = connector.connect(**config)
            if conn.is_connected():
                cursor = conn.cursor()
                if expression is not None \
                        and isinstance(expression, list) \
                        and len(expression) == 2:

                    if expression[1].isdigit():
                        cursor.execute(f'DELETE FROM {db_name}.{db_table} \
                        WHERE {expression[0]}={expression[1]}')
                    else:
                        cursor.execute(f'DELETE FROM {db_name}.{db_table} \
                        WHERE {expression[0]}="{expression[1]}"')
                    conn.commit()
                    return 'Deleted successfully'

                logging.warning('expression value is: %s', expression)
                raise TypeError(f'expression value is:{expression}')

            logging.warning("cannot make the connection with Mysql")
            return None
        return f"Either DataBase {db_name} or Table {db_table} doesn't exist!"
    except TypeError as type_err:
        logging.error('%s: %s', type_err.__class__.__name__, type_err)
        raise
    except connector.ProgrammingError as prog_err:
        logging.error('%s: %s', prog_err.__class__.__name__, prog_err)
        return "Programming error"
    except connector.Error as err:
        logging.error('%s: %s', err.__class__.__name__, err)
        return f"{err.__class__.__name__} error"
    finally:
        conn.close()


def update_db_data(db_name: str = None, db_table: str = None,
                   set_value: List[str] = None, target_exp: List[str] = None):
    """
    This function can be used to update a specific column value of the dataset.
    It requires the set_value=[column name, updated value]
    and target_exp=[column name, value]

    :param db_name: Database name which is containing the data table.
    :param db_table: Data Table name from which the data will be retrieved
    :param set_value: Data to be updated inside the dataset
                        Eg:- set_value=["original_title", "an original title"]

    :param target_exp: column name with value where update needs to be done.
                        Eg:- target_exp=['id', 'id_4']
    :return: msg->"Update successful"
    """
    try:
        if db_name is not None and db_table is not None:
            conn = connector.connect(**config)
            if conn.is_connected():
                cursor = conn.cursor()
                if isinstance(set_value, list) \
                        and len(set_value) == 2 \
                        and isinstance(target_exp, list) \
                        and len(target_exp) == 2:

                    if set_value[1].isdigit() and target_exp[1].isdigit():
                        cursor.execute(f"UPDATE {db_name}.{db_table} SET "
                                       f"{set_value[0]}={set_value[1]} "
                                       f"WHERE {target_exp[0]}={target_exp[1]}")

                    elif set_value[1].isdigit():
                        cursor.execute(f'UPDATE {db_name}.{db_table} SET '
                                       f'{set_value[0]}={set_value[1]} '
                                       f'WHERE {target_exp[0]}="{target_exp[1]}"')

                    elif target_exp[1].isdigit():
                        cursor.execute(f'UPDATE {db_name}.{db_table} SET '
                                       f'{set_value[0]}="{set_value[1]}" '
                                       f'WHERE {target_exp[0]}={target_exp[1]}')

                    else:
                        cursor.execute(f'UPDATE {db_name}.{db_table} SET '
                                       f'{set_value[0]}="{set_value[1]}" '
                                       f'WHERE {target_exp[0]}="{target_exp[1]}"')
                    conn.commit()
                    return "Update successful"

                logging.warning("set_value:%s target_exp:%s", set_value, target_exp)
                return None
            return None
        return None
    except connector.errors.ProgrammingError as prog_err:
        logging.error('%s: %s', prog_err.__class__.__name__, prog_err)
        raise
    except connector.Error as err:
        logging.error('%s: %s', err.__class__.__name__, err)
        raise
    finally:
        conn.close()


def create_db_data(db_name: str = None, db_table: str = None, row_values: dict = None):
    """
    This function can be used to Insert new row inside the database table.
    It takes the dict of column-value where column is from the data table
    and values that needs to be updated
    :param db_name: Database name which is containing the data table.
    :param db_table: Data Table name from which the data will be retrieved
    :param row_values: Takes the dict of column(key)-value(value)
                        pair to be stored in the datatable.
    :return: msg->"Successfully inserted"
    """

    try:
        if db_name is not None and \
                db_table is not None and \
                row_values is not None:
            conn = connector.connect(**config)
            if conn.is_connected():
                cursor = conn.cursor(buffered=True)
                columns = list(row_values.keys())
                columns = ','.join(columns)

                values = [row_values[col] for col in row_values]
                cursor.execute(f'INSERT INTO {db_name}.{db_table} '
                               f'({columns}) VALUES {tuple(values)}')
                conn.commit()
                conn.close()
                return "Successfully inserted"
            logging.warning("cannot make the connection with Mysql")
            return None
        return f"Either DataBase {db_name} or Table {db_table} doesn't exist!"
    except connector.ProgrammingError as prog_err:
        logging.error('%s: %s', prog_err.__class__.__name__, prog_err)
        return f"{prog_err.__class__.__name__}"
    except connector.IntegrityError as intg_err:
        logging.error('%s: %s', intg_err.__class__.__name__, intg_err)
        return "Integrity error"
    except connector.DatabaseError as db_err:
        logging.error('%s: %s', db_err.__class__.__name__, db_err)
        return "Database error"
    except connector.Error as err:
        logging.error('%s: %s', err.__class__.__name__, err)
        return f"{err.__class__.__name__} error"
    except AttributeError as att_err:
        logging.error('%s: %s', att_err.__class__.__name__, att_err)
        return f"{att_err.__class__.__name__}"
